/*
* CCLmod.c - store expanded character classes
*
*
* Entry points:
*
*	cclin(c)
*	char c;
*		Store c in the current character class.  Character classes
*		(and negated character classes) are saved by calling cclnew()
*		to get a character class ID, then storing the characters one
*		at a time by calling cclin(char).
*
*	int cclmem(c, class_id)
*	char c;
*	int class_id;
*		Tell if c is in the identified character class.
*
*	int cclnew()
*		Prepare to record a character class; return the class ID.
*
*
* Implementation:
*
* 	Characters are stored in a single array.  Each class is terminated by
*	the null character.
*/

#define CCLMAX 500		/* limits total sizes of character classes */

static int ccl_next = 0;	/* subscript of next free location */
static char chars[CCLMAX];	/* holds the stored characters */

/* cclin - record a member of the current character class */
cclin(c)
char c;
{
	if (ccl_next >= CCLMAX - 1)	/* -1 for final null character */
		fatal("Too many characters in character classes.");
	chars[ccl_next++] = c;
}

/* cclmem - tell if a character is in the identified character class */
cclmem(c, id)
char c;
int id;
{
	while (chars[id] != '\0')
		if (chars[id++] == c)
			return(1);
	return(0);
}

/* cclnew - return the subscript of the next free location (= ID of class) */
cclnew()
{
	return(++ccl_next);	/* skip null character */
}
