/*
* test.fsm.c - test harness for the finite-state machine builder
*
* Link it with the object files for parse.c, token.c, FSMmod.c and lib.c .
* With the regular expression
*	a*.(bc|d)+\n
* it produces:
*	
	STATE	TYPE	LEFT	RIGHT
	   1:	 a	  2
	   2:	 NIL	  1	  3
	   3:	 NIL	  4
	   4:	 .	  8
	   5:	 b	  6
	   6:	 c	  9
	   7:	 d	  9
	   8:	 NIL	  5	  7
	   9:	 NIL	  10
	   10:	 NIL	  8	  11
	   11:	 NIL	  12
	   12:	 newline
	The start state is 2.
	The final state is 12.
*/

#include "match.h"
#define max(x,y)   (x > y) ? x : y

main(argc, argv)
int argc;
char *argv[];
{
	int i, final, lson, max_state, rson, type, value;

	if (argc != 2)
		fatal("Wrong argument count.");
	parse(argv[1]);
	printf("\nSTATE\tTYPE\tLEFT\tRIGHT\n");

	for (i = max_state = 1; i <= max_state; ++i) {
		state(i, &type, &value, &lson, &rson);
		max_state = max(lson, max_state);
		max_state = max(rson, max_state);
		printf("   %d:\t ", i);
		switch (type) {
			case CHAR:
				if (value == '\n')
					printf("newline");
				else
					putchar(value);
				break;
			case '.':
				putchar('.');
				break;
			case CCL:
				printf("CCL %d", value);
				break;
			case NCCL:
				printf("NCCL %d", value);
				break;
			case NIL:
				printf("NIL");
				break;
			default:
				fatal("Illegal type of machine state.");
		}
		if (lson == NIL) {
			putchar('\n');
			final = i;
		} else if (rson == NIL)
			printf("\t  %d\n", lson);
		else
			printf("\t  %d\t  %d\n", lson, rson);
	}
	state(0, &type, &value, &lson, &rson);
	printf("The start state is %d.\n", lson);
	printf("The final state is %d.\n", final);
}

/* cclin - stub for the procedure that inserts a character in a character class */
cclin(c)
char c;
{
	if (c == '\n')
		printf("newline  ");
	else
		printf("%c  ",c);
}

/* cclnew - stub for the procedure that begins a new character class */
int cclnew()
{
	static int next_ccl = 0;

	printf("\nnew ccl given ID %d; its members are:\n\t", ++next_ccl);
	return(next_ccl);
}
