/*
* test.parse.c - test harness for the regular expression parser
*
* Link it with the object files for token.c, parse.c and lib.c .
* With the regular expression
*	[!]\na]a*.(bc|[d-g])+\n
* it produces:
*
	new ccl given ID 1; its members are:
		]  newline  a  
	1:	NCCL	1
	2:	CHAR	a
	3:	2	*
	4:	1	CAT	3
	5:	.
	6:	4	CAT	5
	7:	CHAR	b
	8:	CHAR	c
	9:	7	CAT	8
	new ccl given ID 2; its members are:
		d  e  f  g  
	10:	CCL	2
	11:	9	|	10
	12:	11	+
	13:	6	CAT	12
	14:	CHAR	newline
	15:	13	CAT	14
*/

#include "match.h"

static int nodes = 0;		/* number of nodes currently in the parse tree */
static int list_ccl = 0;	/* tells if currently listing CCL or NCCL */


main(argc, argv)
int argc;
char *argv[];
{
	if (argc != 2)
		fatal("Needs exactly one argument, the regular expression.");
	parse(argv[1]);
}

/* cclin - stub for the procedure that inserts a character in a character class */
cclin(c)
char c;
{
	if (c == '\n')
		printf("newline  ");
	else
		printf("%c  ", c);
}
 
/* cclnew - stub for the procedure that begins a new character class */
int cclnew()
{
	static int next_ccl = 0;

	printf("new ccl given ID %d; its members are:\n\t", ++next_ccl);
	list_ccl = 1;
	return(next_ccl);
}

/* treelit - stub for the procedure that installs literals in the parse tree */
int treelit(type, value)
int type, value;
{
	if (list_ccl == 1) {
		list_ccl = 0;
		putchar('\n');
	}
	printf("%d:\t", ++nodes);
	switch(type) {
		case (CHAR):
			printf("CHAR\t");
			if (value == '\n')
				printf("newline\n");
			else
				printf("%c\n", value);
			break;
		case('.'):
			printf(".\n");
			break;
		case(CCL):
			printf("CCL\t%d\n", value);
			break;
		case(NCCL):
			printf("NCCL\t%d\n", value);
			break;
		default:
			fatal("treelit(): Illegal node type.");
	}
	return(nodes);
}

/* treeop - stub for the procedure that installs operators in the parse tree */
int treeop(type, lson, rson)
int type, lson, rson;
{
	if (list_ccl == 1) {
		list_ccl = 0;
		putchar('\n');
	}
	printf("%d:\t", ++nodes);
	switch(type) {
		case('|'):
			printf("%d\t%c\t%d\n", lson, type, rson);
			break;
		case(CAT):
			printf("%d\tCAT\t%d\n", lson, rson);
			break;
		case('+'):
		case('*'):
			printf("%d\t%c\n", lson, type);
			break;
		default:
			fatal("treeop(): Illegal node type.");
	}
	return(nodes);
}
